#!/bin/sh

echo "Start Cardinality"

# Extract cardinality constraints
#   1,000 second timeout for direct and 1,000 timeout for encoded constraints
./../cnf2knf/cnf2knf -Direct_timeout 300 --Encoded_AMO=false $1 > card.knf

echo "End Cardinality"

# Output cardinality extraction stats
head -n 30 card.knf

p=$(grep "p knf" card.knf | wc | awk '{print  $1}')
if [ $p -gt 0 ]
then

  ./../knf2cnf card.knf > encoded.cnf
else
  grep -v "c " card.knf > encoded.cnf
fi

echo "Start Cadical"

# Run cadical on the permuted CNF, with a 5000 second timeout
./../kissat/build/kissat encoded.cnf solve.drat --no-binary --time=5000 > cadical.out

echo "End Cadical"


# Check if formula was satisfiable or unsatisfiable
p=$(grep "s SATISFIABLE" cadical.out | wc | awk '{print  $1}')
  
if [ $p -gt 0 ]
then
  # SAT case
  echo "Start SAT"
  # get units from assignment
    grep "^v " cadical.out | tr " " "\n"  | grep -v -e "v" -e "^0" | awk '{print $1" 0"}' > tmp-assignment-units.out
    
    # check if any variables were eliminated
    head -n 30 card.knf | grep "c Eliminated variables IDs:" | tr " " "\n"  | grep -v -e "c" -e "Eliminated" -e "variables" -e "IDs:" > eliminatedVars.txt
    
    p=$( wc eliminatedVars.txt | awk '{print  $1}')
    if [ $p -gt 0 ]
    then # remove eliminated encoding variables from the assignment
      python3 ../process-assignment.py -i tmp-assignment-units.out -e eliminatedVars.txt -o assignment-units.out
    else # no encoding variables eliminated
      mv tmp-assignment-units.out assignment-units.out
    fi
    
    # add units to original CNF formula
    cat $1 assignment-units.out > cnf-with-units.cnf
    # check by running cadical with units
    ./../kissat/build/kissat --relaxed cnf-with-units.cnf
else

  p=$(grep "s UNSATISFIABLE" cadical.out | wc | awk '{print  $1}')
  if [ $p -gt 0 ]
  then
  # UNSAT case
    echo "Start UNSAT Checking"
    
    cat cadical.out
    
    ./../derivation card.knf > deriv.drat
    cat deriv.drat solve.drat > $2/proof.out
    
#    cat deriv.drat solve.drat > proof.drat
#
#    echo "Start Drat Checking"
#    ./../drat-trim/drat-trim $1 proof.drat > drat-check.out
#    cat drat-check.out | grep -v "WARNING"
#    echo "End Drat Checking"

  fi
fi
