# TabularaSAT Solver

This is a clean and generic SAT solver with several variants.

It aims to be both competitive with modern state-of-the-art SAT solvers,
particularly with ['Kissat'](https://github.com/arminbiere/kissat), on
the one hand and on the other hand allows (as one variant) to mimic
['MiniSAT'](https://github.com/niklasso/minisat), which itself had a similar
goal, but in the first decade of this millennium.  Thus the solver allows to
explore both the past and future of high-performance SAT solving.

The ['Satch'](https://github.com/arminbiere/satch) SAT solver followed a
similar goal but with a more didactic focus.  As with
['Satch'](https://github.com/arminbiere/satch) we keep code in one set of
solver source files to avoid redundant data-structures and code when
changing and evaluating those variants.  However, while
['Satch'](https://github.com/arminbiere/satch) is only configured at
compile-time by complex `#ifdef ...` preprocessing commands, we rely for
['TabularaSAT'](https://github.com/arminbiere/tabularasat) on a hybrid
method where all the features can be set at run-time in the
['generic'](src/generic/README.md) variant while still allowing to remove
redundant code during compilation for specific variants, most prominently in
the ['vanilla'](src/vanilla/README.md) variant, which mimics
['MiniSAT'](https://github.com/niklasso/minisat).

We have further factored out some common shared code which sits in the
[`src/shared`](src/shared) directory. The individual solver variants are in
separate directories such as [`src/default`](src/default),
[`src/vanilla`](src/vanilla) etc. but mostly differ in compile time
parameter settings defined in the respective `variant.hpp` files.

## Build

To configure and compile the default solver just run `./configure && make`
and optionally `make test` for testing.  This can also be achieved by going
directly to the [`src/default`](src/default) directory and issue
`./configure && make` there and also optionally `make test` for testing.
The other solver variants [`generic`](src/generic/README.md) and
[`vanilla`](src/vanilla/README.md) can be compiled in the
same way in their directory or by using `./configure --all && make` in the
root directory and optionally `make test` to test them all in parallel.
To compile and test everything in parallel `./configure --all && make -j
test` is recommended (if you have enough cores and memory).

The object and temporary build files are stored in the corresponding build
sub-directory, e.g., `build/default` for the default variant.  Furthermore,
the resulting binaries of the individual solvers are copied from the build
directories where they are compiled into the top-level `bin` directory.  For
instance `bin/tabularasat-default` is the default solver derived from the
sources in [`src/default`](src/default).  It will further be linked to
`bin/tabularasat`, which then can be considered to be the default binary.
