## Source Code

Here is the source code of all the solver variants:

- The [`default`](default) solver is closest to the state-of-the-art and
  has almost all features/options compiled-in such they can be configured
  at run-time.  Features deemed to be too costly, w.r.t. time or space, even
  if disabled and checked at run-time, are left out at compile-time.

- The [`generic`](generic) solver really has all implemented features
  compiled in, even if disabled by default at-run-time. This might still
  degrade performance slightly compared to the default variant.

- The [`vanilla`](vanilla) solver mimics in its default configuration
  MiniSAT 2.2.0 in terms of features and heuristics,  but of course has more
  compact data structures used in TabularaSAT, i.e., virtual binary clauses,
  and faster reduction.  The implementation in `vanilla` aims at trimming
  compiled-in code down by defining corresponding `N...` macros in
  `variant.hpp` and thus also in `dependencies.hpp` as much as possible.

The `generic` variant still has a `--vanilla` configuration which can be
selected at run-time and should behave identical to the trimmed down
`vanilla` code but is slightly slower. Thus, for evaluations, you might want
to use the actual `tabularasat-vanilla` solver and not the generic version
with this configuration, i.e., `taularasat-generic --vanilla`.

The same argument applies to all the configurations which are variants of
the `vanilla` variant and only need the code included in the `vanilla`
solver, i.e., `core` (same as `vanilla` but without pre- nor inprocessing),
`parfait` (inprocessing instead of preprocessing), `semifreddo` (both
preprocessing and inprocessing), and `straciatella` (as `semifreddo` but
with transitive reduction and equivalent literal substitution too).

Non-solver specific shared code is placed into [`shared`](shared).  This
code does not depend on the actual solver compiled (does for instance not
need to know the `Solver` name) but is still compiled separately for each
variant in order to allow different compile-time configurations.

The actual solver source code sits in the [`generic`](generic) directory and
solver variants just link to it, more precisely to the parts compiled in for
that variant (e.g., there is no `vivify.cpp` in `vanilla`).

As the actual solver source code is shared and still needs to be
instantiated there are compile time options (macros) in the individual
`variant.hpp` files.  They define which part of the linked-in solver code is
used and compiled.  Thus object files for each solver variant differ and are
placed in separate sub-directories in the `build` directory.

To configure and build the solver go to that individual solver directory
and call `./configure && make` followed optionally by `make test`.  You can
also do the same in the root directory using `./configure --all && make` to
configure and build all variants and not just the `default` variant.
