## TabularaSAT Default

This is the `default` (almost generic) variant of TabularaSAT.  It has the
goal to be as close as possible to a state-of-the-art solver such as Kissat
while at the same time keeping code size small and still being extensively
configurable at run-time.

Compared to the `generic` version this `default` version does not compile in
some code which might risks to negatively impact performance both in terms
of memory usage and solving speed if included.  See 'variant.hpp' for macros
listing those permanently disabled features.

Compared to the `light' version of Kissat (which in turn is already stripped
down from `kissat-sc2022-bulky`) it does not have an embedded SAT solver
(Kitten) and thus no SAT sweeping nor semantic gate detection.  It also
lacks the fast backbone computation on the binary implication graph.  However
it has a more flexible scheduling of inprocessing and also does vivification
on irredundant clauses.  The arena for storing watcher stack is simpler and
faster than in Kissat but needs slightly more memory.  On the other hand
Kissat distinguishes between internal and external variable indices and thus
allow a compact internal memory layout by removing eliminated, substituted
and root-level fixed variables from further consideration during solving.

Use `./configure && make` for optimizing build process and `./configure
--debug && make` if you want to include symbols and disable assertion
checking.  If you compiled with debugging support then you can also log
internal execution by using `--logging` as command line option to `solver`.
The resulting binary is produced as `../../build/default/binary` and then
copied to `../../bin/tabularasat-default` and linked to `./solver`.

There are also some regression tests included and `make test` executes them
and should not produce any failures.
