#ifndef _tabularasat_inline_hpp_INCLUDED
#define _tabularasat_inline_hpp_INCLUDED

#include <cassert>
#include <cmath>
#include <cstdint>

namespace TabularaSAT {

inline double average (double a, double b) { return b ? a / b : 0; }

inline double percent (double a, double b) { return average (100 * a, b); }

inline bool is_power_of_two (uint64_t n) { return n && !(n & (n - 1)); }

inline double logn (uint64_t count) {
  assert (count);
  return log10 (count + 9);
}

inline double logpow (uint64_t count, int exponent) {
  double res = 1;
  if (exponent) {
    double factor = logn (count);
    if (exponent > 0)
      for (int i = 0; i != exponent; i++)
        res *= factor;
    else
      for (int i = 0; i != exponent; i--)
        res /= factor;
  }
  return res;
}

inline double nlogpow (uint64_t count, int exponent) {
  return count * logpow (count, exponent);
}

inline void increment_saturating (unsigned &x) { x += !!~x; }

struct rank_less_unsigned {
  unsigned operator() (unsigned u) { return u; }
};

}; // namespace TabularaSAT

#endif
