#ifndef _tabularasat_references_hpp_INCLUDED
#define _tabularasat_references_hpp_INCLUDED

#include "solver.hpp"

#include <cassert>

namespace TabularaSAT {

inline bool tagged (Reference ref) { return ref & 1; }
inline Position untag (Reference ref) { return ref >> 1; }

inline Reference tag_binary_or_large (Position d, unsigned t) {
  assert (t == 0 || t == 1);
  Reference res = (d << 1) | t;
  assert (tagged (res) == t);
  assert (untag (res) == d);
  assert (res != UNIT_REASON);
  assert (res != DECISION_REASON);
  return res;
}

inline Reference tag_binary (unsigned other) {
  assert (other <= MAXIMUM_INTERNAL_LITERAL);
  return tag_binary_or_large (other, 1);
}

inline Reference tag_large (Position pos) {
  assert (pos <= MAXIMUM_POSITION_OF_CLAUSE);
  return tag_binary_or_large (pos, 0);
}

inline bool binary_reference (Reference ref) {
  return ref != DECISION_REASON && ref != UNIT_REASON && tagged (ref);
}

inline bool large_reference (Reference ref) {
  return ref != DECISION_REASON && ref != UNIT_REASON && !tagged (ref);
}

inline Clause *Solver::dereference_clause (Reference ref) {
  assert (!tagged (ref));
  unsigned i = untag (ref);
  return clauses[i];
}

inline Reference Solver::reference_clause (Clause *c) {
  return tag_large (clauses.position (c));
}

inline References *Solver::references (unsigned lit) {
  assert (lit < literals);
  return &matrix[lit];
}

// With 'typedef std::vector<Reference> References' we could just use
// 'references (lit).push_back (element)' instead of the following
// functions.  However, our 'References' stacks are allocated in the
// 'Solver' references arena ('matrix') and might move during pushing
// elements.  Therefore this function needs to have access to that arena,
// conveniently as a member function of 'Solver' instead of 'Matrix', and
// further also needs to update the pointer to the references stack given as
// second argument in case it is reused in the calling function.

// We require the literal argument as the 'refs' pointer might point to the
// shared empty references stack at the beginning of the arena with invalid
// literal field and after obtaining a new references stack needs to be set.

// The last argument 'expect_another' tells the memory allocator in
// 'enlarge_references' that we will push another reference to the same
// references stack of the literal next and thus its capacity should be
// fixed to accommodate the 'element' and another reference.

inline void Solver::push_reference (unsigned lit, References *&refs,
                                    Reference element,
                                    bool expect_another) {
  assert (refs == references (lit));
  if (refs->size () + 1 + expect_another > refs->capacity ())
    refs = enlarge_references (lit, refs, expect_another);
  assert (refs->_size < refs->_capacity);
  refs->data[refs->_size++] = element;
}

inline void Solver::push_reference (unsigned lit, Reference ref,
                                    bool expect_another) {
  References *refs = &matrix[lit];
  push_reference (lit, refs, ref, expect_another);
}

}; // namespace TabularaSAT

#endif
