#include "solver.hpp"

namespace TabularaSAT {

inline void Solver::unassign (unsigned lit) {
  debug ("unassign %s", debug_literal (lit));
  unsigned not_lit = negate (lit);
  assert (values[lit] > 0);
  assert (values[not_lit] < 0);
  values[lit] = values[not_lit] = 0;
  unassigned++;
  assert (unassigned);
  unsigned idx = index (lit);
#ifndef NVIVIFY
  if (!vivifying)
#endif
#ifndef NPROBE
    if (!probing)
#endif
    {
#ifndef NSWITCH
      if (!stable)
        queue.update (idx);
      else
#endif
      {
        if (!bumper.contains (idx))
          bumper.push (idx);
      }
    }
#ifndef QUIET
  if (reasons[idx] == DECISION_REASON && !flags[idx].active) {
    assert (assigned_eliminated);
    assigned_eliminated--;
  }
#endif
}

void Solver::backtrack (unsigned jump) {
  assert (jump < level);
  assert (control.size () == level + 1);
  debug ("backtracking to level %u", jump);

  size_t begin = control[jump + 1].trail;
  size_t end = trail.size ();
  size_t i = begin, j = i;

  while (i != end) {
    unsigned lit = trail[i++];
#ifndef NCHRONO
    unsigned lit_level = levels[index (lit)];
    if (lit_level > jump)
      unassign (lit);
    else {
      unsigned idx = index (lit);
      position[idx] = j;
      trail[j++] = lit;
      debug ("keeping out-of-order assigned %s", debug_literal (lit));
#ifndef QUIET
      if (!jump && !lit_level) {
        assert (out_of_order_units);
        out_of_order_units--;
      }
#endif
    }
#else
    assert (levels[index (lit)] > jump);
    unassign (lit);
#endif
  }

  trail.resize (j);
  trail.propagated = trail.begin () + begin;
  control.resize (jump + 1);
  level = jump;
}

bool Solver::backtrack_to_root_level_and_propagate () {
  assert (level);
  debug ("forced to backtrack to root-level");
  backtrack (0);
  if (search_propagate ())
    return true;
  debug ("root level propagation yields conflict");
  inconsistent = true;
  trace_empty ('a');
  return false;
}

}; // namespace TabularaSAT
