#include "solver.hpp"

#include <cinttypes>
#include <cstdarg>

namespace TabularaSAT {

#ifdef LOGGING

void Solver::debug_clause (Clause *c, const char *fmt, ...) {
  if (!logging ())
    return;
  debug_prefix ();
  va_list ap;
  va_start (ap, fmt);
  vprintf (fmt, ap);
  va_end (ap);
  if (c->redundant) {
    printf (" redundant");
#ifndef NGLUE
    printf (" glue %u", c->glue);
#endif
#ifndef NUSED
    printf (" used %u", c->used);
#endif
#ifndef NACTIVITY
    printf (" activity %g", (double) c->activity.as_float);
#endif
  } else
    fputs (" irredundant", stdout);
  if (c->garbage)
    fputs (" garbage", stdout);
  if (c->shortened)
    fputs (" shortened", stdout);
  size_t pos = clauses.position (c);
  printf (" size %u clause[%" PRIu64 "]@%zu", c->size, c->id, pos);
  for (auto lit : *c)
    printf (" %s", debug_literal (lit));
  debug_suffix ();
}

void Solver::debug_binary (unsigned lit, unsigned other, const char *fmt,
                           ...) {
  if (!logging ())
    return;
  debug_prefix ();
  va_list ap;
  va_start (ap, fmt);
  vprintf (fmt, ap);
  va_end (ap);
  printf (" binary clause %s %s", debug_literal (lit),
          debug_literal (other));
  debug_suffix ();
}

void Solver::debug_clauses () {
  for (auto c : clauses)
    debug_clause (c, "clauses[%zu]", clauses.position (c));
}

#endif

}; // namespace TabularaSAT
