#include "references.hpp"
#include "solver.hpp"

namespace TabularaSAT {
bool Solver::minimize_literal (unsigned lit, unsigned depth) {
  if (depth > OPTION (minimization_depth))
    return false;
  unsigned idx = index (lit);
  Flags &f = flags[idx];
  if (depth && f.seen)
    return true;
  if (f.poisoned)
    return false;
  if (f.minimizable)
    return true;
  Reference reason = reasons[idx];
  if (reason == DECISION_REASON)
    return false;
  if (reason == UNIT_REASON)
    return true;
  bool res = true;
  if (tagged (reason))
    res = minimize_literal (untag (reason), depth + 1);
  else {
    INC_COUNT (search_ticks);
    Clause *c = dereference_clause (reason);
    for (auto other : *c)
      if (other != lit && !minimize_literal (other, depth + 1)) {
        res = false;
        break;
      }
  }
  if (res) {
    f.minimizable = true;
    minimizable.push_back (idx);
  } else {
    f.poisoned = true;
    poisoned.push_back (idx);
  }
  return res;
}

void Solver::reset_minimized_and_poisoned () {
  for (auto idx : minimizable)
    flags[idx].minimizable = false;
  for (auto idx : poisoned)
    flags[idx].poisoned = false;
  minimizable.clear ();
  poisoned.clear ();
}

void Solver::minimize_deduced_clause () {
  start ("minimize", 4);
  auto begin = clause.begin ();
  auto end = clause.end ();
  auto p = begin, q = p;
  for (p = begin; p != end; p++)
    if (minimize_literal (*p)) {
#ifndef QUIET
      statistics.minimized_literals++;
#endif
    } else
      *q++ = *p;
  clause.resize (q - begin);
  debug_vector (clause, "minimized first UIP clause");
  reset_minimized_and_poisoned ();
  stop ("minimize");
}

}; // namespace TabularaSAT
