// vi:tw=0
#ifndef _tabularasat_options_hpp_INCLUDED
#define _tabularasat_options_hpp_INCLUDED

#include "shared.hpp"
#include "variant.hpp"

// clang-format off

namespace TabularaSAT {

// Internal list of alphabetically ordered OPTION ()

#ifndef NOPTIONS

struct SolverOptions : Options {

  Option *begin () { return &first_dummy_option + 1; }
  Option first_dummy_option = Option ("first-dummy-option", 0, 0, 0, "first dummy option");

#define OPTION(SYMBOL, NAME, DEFAULT, MIN, MAX, DESCRIPTION) \
  Option SYMBOL = Option (NAME, DEFAULT, MIN, MAX, DESCRIPTION)

#else

#define OPTION(SYMBOL, NAME, DEFAULT, MIN, MAX, DESCRIPTION) \
  static const unsigned option_ ## SYMBOL = (unsigned) (DEFAULT)

#endif

#ifndef NACTIVITY
  OPTION (activity_relevant, "activity-relevant", 0, 0, 1, "use activity to determine redundant clause relevance");
#endif
#ifndef NALLUIPS
  OPTION (all_uips, "all-uips", 1, 0, 1, "after learning a unit learn all UIPs");
#endif
#if !defined(NCHRONO) || !defined(NOTFS)
  OPTION (backjump_limit, "backjump-limit", 100, 0, INT_MAX, "non-chronological and on-the-fly subsumption backjumping limit");
#endif
  OPTION (bump_learned, "bump-learned", 0, 0, 1, "bump learned clause initially");
#ifndef NBUMPREASONS
  OPTION (bump_reason_side_in_focused_mode, "bump-reason-side-in-focused-mode", 0, 0, 1, "enable in focused mode too");
#ifndef NSWITCH
  OPTION (bump_reason_side_in_stable_mode, "bump-reason-side-in-stable-mode", 1, 0, 1, "enable in stable mode too");
#endif
  OPTION (bump_reason_side_limit, "bump-reason-side-limit", 10, 1, 10000, "factor limit on number additionally bumped");
  OPTION (bump_reason_side, "bump-reason-side", 1, 0, 1, "bump reason side variables");
  OPTION (bump_reason_side_rate, "bump-reason-side-rate", 10, 1, 10000, "decision-rate reason-side bumping limit");
#endif
#ifndef NCHRONO
  OPTION (chronological_backtracking, "chronological-backtracking", 1, 0, 1, "enable chronological backtracking");
#endif
#ifndef NACTIVITY
  OPTION (clause_activity_alpha, "clause-activity-alpha", 1000, 1, 100000, "clause scores decay (alpha) in per millions");
#endif
#ifndef NELIMINATIONHEAP
  OPTION (eliminate_irrelevant, "eliminate-irrelevant", 1, 0, 1, "focus elimination attempts on irrelevant variables");
#endif
  OPTION (eliminate_variables, "eliminate-variables", 1, 0, 1, "enable variable elimination");
  OPTION (elimination_effort, "elimination-effort", 200, 1, 1000, "relative variable elimination effort in per mille");
  OPTION (elimination_garbage_limit, "elimination-garbage-limit", 100, 1, 1000, "garbage collection limit during variable elimination in percent");
#ifndef NELIMINATIONHEAP
  OPTION (elimination_heap, "elimination-heap", 1, 0, 1, "use priority queue for variable elimination");
#endif
  OPTION (elimination_interval, "elimination-interval", 500, 10, INT_MAX, "conflicts between variable eliminations");
  OPTION (elimination_occurrence_limit, "elimination-occurrence-limit", 100, 0, INT_MAX, "one-sided occurrence limit");
  OPTION (elimination_rounds, "elimination-rounds", 2, 1, INT_MAX, "number of elimination rounds per elimination");
#ifndef NELIMINATIONHEAP
  OPTION (elimination_scores, "elimination-scores", 1, 0, 1, "uses occurrence based scores for elimination");
  OPTION (elimination_scoring, "elimination-scoring", 3, 0, 3, "0=p+n, 1=p*n, 2=p*n+p+n, 3=p*n-p-n");
#endif
#ifndef NGATES
  OPTION (extract_and_gates, "extract-and-gates", 1, 0, 1, "extract AND gates during variable elimination");
  OPTION (extract_equivalences, "extract-and-gates", 1, 0, 1, "extract equivalences during variable elimination");
  OPTION (extract_gates, "extract-gates", 1, 0, 1, "extract gates during variable elimination");
#endif
#ifndef NPROBE
  OPTION (failed_literal_effort, "failed-literal-effort", 10, 1, 1000, "relative failed literal probing effort in per mille");
  OPTION (failed_literal_interval, "failed-literal-interval", 50, 2, INT_MAX, "conflicts between failed literal probings");
  OPTION (failed_literal_probing, "failed-literal-probing", 0, 0, 1, "enable failed literal probing");
  OPTION (failed_literal_preprocessing, "failed-literal-preprocessing", 10, 1, 1000,
          "initial failed literal probing during preprocessing minimum-effort factor");
#endif
#ifndef NSWITCH
  OPTION (filtered_restart_interval, "filtered-restart-interval", 1, 1, INT_MAX, "conflicts between filtered restarts");
#endif
#ifndef NGLUE
  OPTION (focused_restart_filter, "focused-restart-filter", 1, 0, 1, "moving averages based restart filtering in focused mode");
  OPTION (glue_relevant, "glue-relevant", 1, 0, 1, "use glucose level (LBD) to determine redundant clause relevance");
#endif
  OPTION (increase_factor, "increase-factor", 150, 101, 200, "kept learned clauses increase interval increase in percent (vanilla)");
  OPTION (initial_elimination_limit, "initial-elimination-limit", 500, 0, INT_MAX, "conflicts until first variable eliminations");
  OPTION (initial_increase, "initial-increase", 100, 10, INT_MAX, "initial conflict interval to increase kept learned clause (vanilla)");
#ifndef NPROBE
  OPTION (initial_failed_literal_limit, "initial-failed-literal-limit", 50, 0, INT_MAX, "ocnflicts until first failed literal probing");
#endif
  OPTION (initial_learned_fraction, "initial-learned-fraction", 33, 1, 100, "percentage initial kept learned clauses (vanilla)");
#ifndef NSWITCH
  OPTION (initial_mode_stable, "initial-mode-stable", 0, 0, 1, "start solving in stable and not focused mode");
#endif
  OPTION (initial_phase, "initial-phase", 1, 0, 1, "default initial phase assigned to variables");
#ifndef NSUBSTITUTE
  OPTION (initial_substitution_limit, "initial-substitution-limit", 500, 0, INT_MAX, "conflicts until first equivalent literal substitution");
#endif
  OPTION (initial_subsumption_limit, "initial-subsumption-limit", 500, 0, INT_MAX, "conflicts until first clause subsumption");
#ifndef NTRANSITIVE
  OPTION (initial_transitive_limit, "initial-transitive-limit", 50, 0, INT_MAX, "conflicts until first transitive reduction");
#endif
#ifndef NVIVIFY
  OPTION (initial_vivification_limit, "initial-vivification-limit", 100, 0, INT_MAX, "conflicts until first clause vivification");
#endif
  OPTION (inprocessing, "inprocessing", 1, 0, 1, "enable or disable all inprocessing");
#ifndef NJUMPREASONS
  OPTION (jump_reasons, "jump-reasons", 1, 0, 1, "implicit hyper binary resolution for binary reasons");
#endif
#ifndef NGLUE
  OPTION (keep_tier1, "keep-tier1", 1, 0, 1, "keep tier1 learned clauses");
#endif
#ifndef NUSED
  OPTION (keep_used, "keep-used", 1, 0, 1, "keep used learned clauses");
#endif
  OPTION (learned_factor, "learned-factor", 110, 101, 200, "kept learned clauses increase in percent (vanilla)");
  OPTION (maximum_elimination_bound, "maximum-elimination-bound", 16, 0, 1024, "maximum additional clauses during variable elimination");
  OPTION (minimization_depth, "minimization-depth", 1000, 1, INT_MAX, "maximum recursive minimization depth");
  OPTION (minimize_learned_clause, "minimize-learned-clause", 1, 0, 1, "minimize deduced 1st UIP learned clause");
  OPTION (minimum_effort, "minimum-effort", 1e6, 0, INT_MAX, "minimum effort in terms of ticks");
  OPTION (minimum_learned, "minimum-learned", 0, 0, INT_MAX, "minimum number kept learned clauses (vanilla)");
#ifndef NSWITCH
  OPTION (mode_switching_interval, "mode-switching-interval", 1000, 1, INT_MAX, "conflicts between mode switching");
#endif
  OPTION (move_redundant_clauses, "move-redundant-clauses", 1, 0, 1, "move redundant clauses to the end of the arena");
#ifndef NOTFS
  OPTION (on_the_fly_strengthen, "on-the-fly-strengthen", 1, 0, 1, "on-the-fly self-subsuming resolution during analysis");
#endif
  OPTION (preprocessing, "preprocessing", 1, 0, 1, "additional initial and more thorough preprocessing");
  OPTION (preprocessing_rounds, "preprocessing-rounds", 2, 1, INT_MAX, "number rounds of individual preprocessors during initial preprocessing");
#ifndef NPROMOTE
  OPTION (promote_clauses, "promote-clauses", 1, 0, 1, "promote glue of redundant clauses during conflict analysis");
#endif
  OPTION (radix_sort_limit, "radix-sort-limit", 32, 0, INT_MAX, "lower bound on when to do radix instead of generic sort");
  OPTION (reduce_fraction, "reduce-fraction", 75, 0, 100, "percentage of reduced redundant clause");
  OPTION (reduce_interval, "reduce-interval", 1000, 1, INT_MAX, "conflicts between reduction");
  OPTION (reduce, "reduce", 1, 0, 1, "reduction (forgetting) of learned clauses");
#ifndef VANILLA
  OPTION (reduce_vanilla, "reduce-vanilla", 0, 0, 1, "vanilla (MiniSAT) alike reduction scheduler");
#endif
  OPTION (references_garbage_limit, "references-garbage-limit", 40, 10, 1000, "references garbage before collection in percent");
  OPTION (references_garbage_minimum, "references-garbage-minimum", 1<<20, 0, INT_MAX, "absolute garbage minimum before collection");
  OPTION (reluctant_restart_interval, "reluctant-restart-interval", 1024, 1, INT_MAX, "conflicts between reluctant restarts");
#ifndef NREPHASE
  OPTION (rephase_best, "rephase-best", 1, 0, 1, "reset phases to best phase");
  OPTION (rephase_interval, "rephase-interval", 1000, 1, 100000, "conflicts between resetting phases");
  OPTION (rephase_inverted, "rephase-inverted", 1, 0, 1, "reset phases to inverted original phase");
  OPTION (rephase, "rephase", 1, 0, 1, "reset phases in regular intervals");
  OPTION (rephase_original, "rephase-original", 1, 0, 1, "reset phases to original phase");
#endif
#ifndef NWALK
  OPTION (rephase_walk, "rephase-walk", 1, 0, 1, "reset phases through local search");
#endif
  OPTION (resolvent_size, "resolvent-size", 1000, 2, INT_MAX, "maximum resolvent size");
#ifndef NGLUE
  OPTION (restart_margin, "restart-margin", 10, 0, 100, "restart margin in percent fast-over-slow glue");
#endif
  OPTION (restart, "restart", 1, 0, 1, "schedule restarts in regular intervals");
#ifndef NREUSETRAIL
#ifndef NSWITCH
  OPTION (reuse_trail_focused_mode, "reuse-trail-in-focused-mode", 1, 0, 1, "reuse the trail in focused mode");
#endif
  OPTION (reuse_trail, "reuse-trail", 0, 0, 1, "try to reuse the trail during restart");
#ifndef NSWITCH
  OPTION (reuse_trail_stable_mode, "reuse-trail-in-stable-mode", 1, 0, 1, "reuse the trail in stable mode");
#endif
#endif
#ifndef NSHRINK
  OPTION (shrink_learned_clause, "shrink-learned-clause", 1, 0, 1, "shrink deduced learned clause instead of minimizing it");
#endif
#ifndef NSIZERELEVANT
  OPTION (size_relevant, "size-relevant", 0, 0, 1, "use size to determine redundant clause relevance");
#endif
#ifndef NSWITCH
  OPTION (stable_bias, "stable-bias", 10, 1, 1000, "initial bias for stable mode as second mode");
#endif
#ifndef NGLUE
  OPTION (stable_restart_filter, "stable-restart-filter", 0, 0, 1, "moving averages based restart filtering in stable mode");
#endif
  OPTION (strengthen_clauses, "strengthen-clauses", 1, 0, 1, "enable clause self-subsuming resolution during subsumption");
#ifndef NSUBSTITUTE
  OPTION (substitute_variables, "substitute-variables", 1, 0, 1, "enable equivalent literal substitution");
  OPTION (substitution_interval, "substitution-interval", 500, 2, INT_MAX, "conflicts between equivalent literal substitutions");
  OPTION (substitution_rounds, "substitution-rounds", 2, 1, INT_MAX, "number of rounds per substitution");
#endif
  OPTION (subsume_clauses, "subsume-clauses", 1, 0, 1, "enable clause subsumption inprocessing");
  OPTION (subsumed_clause_size, "subsumed-clause-size", 1000, 3, INT_MAX, "maximum size of clause checked to be subsumed");
  OPTION (subsumption_effort, "subsumption-effort", 200, 1, 1000, "relative subsumption elimination effort in per mille");
  OPTION (subsumption_interval, "subsumption-interval", 500, 10, INT_MAX, "conflicts between clause subsumption");
  OPTION (subsumption_occurrence_limit, "subsumption-occurrence-limit", 100, 0, INT_MAX, "one-sided occurrence limit");
  OPTION (subsumption_rounds, "subsumption-rounds", 2, 1, INT_MAX, "number of subsumption rounds per subsumption");
#ifndef NSWITCH
  OPTION (switch_search_mode, "switch-search-mode", 1, 0, 1, "switch between stable and focused search mode");
#endif
#ifndef NTARGET
  OPTION (target_phases, "target-phases", 1, 0, 1, "update and use target phases");
#endif
#ifndef NGLUE
  OPTION (tier1_max_glue, "tier1-max-glue", 2, 0, INT_MAX, "tier1 glucose level for always kept clauses");
  OPTION (tier2_max_glue, "tier2-max-glue", 6, 0, INT_MAX, "tier2 glucose level for clauses with second chance");
#endif
#ifndef NTRANSITIVE
  OPTION (transitive_effort, "transitive-reduction-effort", 20, 1, 1000, "transitive reduction effort in per mille");
  OPTION (transitive_interval, "transitive-reduction-interval", 50, 1, INT_MAX, "transitive reduction conflict interval");
  OPTION (transitive_preprocessing, "transitive-preprocessing", 10, 1, 1000, "initial transitive reduction during preprocessing minimum-effort factor");
  OPTION (transitive_reduction, "transitive-reduction", 1, 0, 1, "transitive reduction of binary implication graph");
#endif
  OPTION (trigger_elimination, "trigger-elimination", 1, 0, 1, "wait for next reduction to trigger elimination");
  OPTION (trigger_vivification, "trigger-vivification", 1, 0, 1, "wait for next reduction to trigger vivification");
  OPTION (variable_score_alpha, "variable-score-alpha", 50, 1, 500, "VSIDS scores decay (alpha) in per mille");
#ifndef NVIVIFY
  OPTION (vivification_effort, "vivification-effort", 100, 1, 1000, "relative redundant clause vivification effort in per mille");
  OPTION (vivification_interval, "vivification-interval", 100, 2, INT_MAX, "conflicts between clause vivifications");
  OPTION (vivification_preprocessing, "vivification-preprocessing", 10, 1, 1000, "initial vivification during preprocessing minimum-effort factor");
  OPTION (vivify_clauses, "vivify-clauses", 1, 0, 1, "enable clause vivification");
  OPTION (vivify_irredundant_clauses, "vivify-irredundant-clauses", 1, 0, 100, "relative irredundant (tier0) clause vivification effort");
  OPTION (vivify_irredundant_limit, "vivify-irredundant-limit", 10, 0, 1000, "percentage of redundant clauses before irredundant vivification");
  OPTION (vivify_tier1_clauses, "vivify-tier1-clauses", 1, 0, 100, "relative tier1 clause vivification effort");
  OPTION (vivify_tier2_clauses, "vivify-tier2-clauses", 1, 0, 100, "relative tier2 clause vivification effort");
#endif
#ifndef NWALK
  OPTION (walking_effort, "walking-effort", 50, 1, 1000, "relative local search effort in per mille");
  OPTION (walking_fit, "walking-fit", 0, 0, 1, "try to to fit CB value of ProbSAT");
#endif
  OPTION (wasted_accessed, "wasted-accessed", 50, 0, 10000, "wasted irredundant clauses access fraction in percent before moving");

#ifndef NOPTIONS

  Option last_dummy_option = Option ("last-dummy-option", 0, 0, 0, "last dummy option");
  Option *end () { return &last_dummy_option; }
};

#undef OPTION
#define OPTION(SYMBOL) options.SYMBOL
#define SOLVER_OPTION(SYMBOL) solver.options.SYMBOL

#else

#undef OPTION
#define OPTION(SYMBOL) option_ ## SYMBOL
#define SOLVER_OPTION OPTION

#endif


};

// clang-format on

#endif
