#include "solver.hpp"

namespace TabularaSAT {

void Solver::update_phases () {
  assert (level);
#ifndef NSWITCH
  if (!stable)
    return;
#endif
  size_t consistently_assigned = control[level].trail;
  bool update_best_assignment = OPTION (rephase) && OPTION (rephase_best) &&
                                (consistently_assigned > best_assigned);
  bool update_target_assignment =
#ifndef NTARGET
      OPTION (target_phases) && (consistently_assigned > target_assigned);
#else
      false;
#endif
  if (!update_best_assignment && !update_target_assignment)
    return;
  size_t begin = trail_root_level ();
  assert (begin <= consistently_assigned);
  for (size_t i = begin; i != consistently_assigned; i++) {
    unsigned lit = trail[i];
    Phase &phase = phases[index (lit)];
    signed char value = sign (lit) ? -1 : 1;
    if (update_best_assignment)
      phase.best = value;
#ifndef NTARGET
    if (update_target_assignment)
      phase.target = value;
#endif
  }
  if (update_best_assignment) {
    best_assigned = consistently_assigned;
    message (2, "updated best assigned to %u", best_assigned);
#ifndef QUIET
    statistics.updated_best_phases++;
#endif
  }
#ifndef NTARGET
  if (update_target_assignment) {
    target_assigned = consistently_assigned;
    message (2, "updated target assigned to %u", target_assigned);
#ifndef QUIET
    statistics.updated_target_phases++;
#endif
  }
#endif
}

}; // namespace TabularaSAT
