#include "references.hpp"
#include "solver.hpp"

namespace TabularaSAT {

// Remove a reference to a large clause from a watcher stack.

void Solver::unwatch_literal (unsigned lit, Reference ref) {
  assert (watching);
  auto watches = references (lit);
  auto begin = watches->begin ();
  auto end = watches->end ();
  auto p = begin, q = p;
  Reference first = INVALID_REFERENCE;
  while (p != end) {
    first = *q++ = *p++;
    if (tagged (first))
      continue;
    Reference second = *q++ = *p++;
    if (second != ref)
      continue;
    q -= 2;
    break;
  }
  assert (q + 2 == p);
  while (p != end)
    *q++ = *p++;
  watches->resize (q - begin);
  assert (first != INVALID_REFERENCE);
  debug_reference (ref, "unwatching %s with blocking literal %s in",
                   debug_literal (lit), debug_literal (untag (first)));
}

// Remove one virtual binary clause watch.

void Solver::unwatch_binary (unsigned lit, unsigned other) {
  Reference search = tag_binary (other);
  assert (watching);
  auto watches = references (lit);
  auto begin = watches->begin ();
  auto end = watches->end ();
  auto p = begin, q = p;
  while (p != end) {
    Reference ref = *p++;
    if (ref == search)
      break;
    *q++ = ref;
    if (!tagged (ref))
      *q++ = *p++;
  }
  assert (q + 1 == p);
  while (p != end)
    *q++ = *p++;
  watches->resize (q - begin);
  debug_binary (lit, other, "unwatched %s in", debug_literal (lit));
}

void Solver::unwatch_clause (Clause *c) {
  assert (watching);
  Reference ref = reference_clause (c);
  unwatch_literal (c->literals[0], ref);
  unwatch_literal (c->literals[1], ref);
}

}; // namespace TabularaSAT
