#ifndef _tabularasat_radix_hpp_INCLUDED
#define _tabularasat_radix_hpp_INCLUDED

#include <cassert>
#include <cstring>
#include <vector>

namespace TabularaSAT {

template <class Element, class Size, class Rank, class Ranker>
void radix_sort_generic (std::vector<Element> &vector,
                         std::vector<char> &sorter, Ranker ranker) {
  assert (vector.size () <= ~(Size) 0);
  const Size size = vector.size ();
  if (size < 2)
    return;
  sorter.resize (size * sizeof (Element));
  Element *tmp = (Element *) &sorter[0];
  Element *dst = tmp, *src = &vector[0];
  Rank lower = ~0u, higher = 0;
  Size count[256];
  for (unsigned i = 0; i != 8 * sizeof (Rank); i += 8) {
    Size from, to;
    if (i) {
      Rank shifted_lower = lower >> i;
      Rank shifted_higher = higher >> i;
      if (shifted_lower == shifted_higher)
        break;
      from = shifted_lower & 255;
      to = shifted_higher & 255;
      if (from == to)
        continue;
    } else {
      from = 0;
      to = 255;
    }
    memset (count + from, 0, (to - from + 1) * sizeof (Size));
    bool sorted = false;
    Rank last_rank = 0;
    for (Size j = 0; j != size; j++) {
      Element element = src[j];
      Rank rank = ranker (element);
      if (!i) {
        lower &= rank;
        higher |= rank;
      }
      rank >>= i;
      rank &= 255;
      count[rank]++;
      if (!sorted)
        continue;
      if (rank > last_rank)
        sorted = false;
      else
        last_rank = rank;
    }
    if (sorted)
      continue;
    if (!i) {
      from = lower & 255;
      to = higher & 255;
      if (from == to)
        continue;
    }
    Size pos = 0;
    for (Size j = from; j <= to; j++) {
      Size delta = count[j];
      count[j] = pos;
      pos += delta;
    }
    for (Size j = 0; j != size; j++) {
      Element element = src[j];
      Rank rank = ranker (element);
      rank >>= i;
      rank &= 255;
      pos = count[rank]++;
      dst[pos] = element;
    }
    std::swap (src, dst);
  }
  if (src == tmp)
    memcpy (dst, tmp, size * sizeof (Element));
#ifndef NDEBUG
  for (Size i = 1; i != size; i++)
    assert (ranker (vector[i - 1]) <= ranker (vector[i]));
#endif
}

}; // namespace TabularaSAT

#endif
