#include "solver.hpp"

namespace TabularaSAT {

unsigned Solver::decide_variable_in_stable_mode () {
#ifndef NSWITCH
  assert (stable);
#endif
  if (bumper.capacity () < variables) {
    bumper.resize (variables);
    size_t pushed = 0;
    for (auto idx : variables)
      if (flags[idx].active)
        bumper.push (idx), pushed++;
    message (2, "initially pushed %zu variables", pushed);
  }
  unsigned decision;
  while (values[literal (decision = *bumper)])
    bumper.pop ();
  debug ("highest scored unassigned variable %s with score %g",
         debug_variable (decision), bumper.score (decision));
  return decision;
}

#ifndef NSWITCH

unsigned Solver::decide_variable_in_focused_mode () {
  assert (!stable);
  if (queue.capacity () < variables) {
    queue.resize (variables);
    size_t enqueued = 0;
    for (auto idx : variables)
      if (flags[idx].active)
        queue.enqueue (idx, true), enqueued++;
    message (2, "initially enqueued %zu variables", enqueued);
  }
  unsigned decision;
  while (values[literal (decision = *queue)])
    queue.pop ();
  debug ("last enqueued unassigned variable %s with stamp %u",
         debug_variable (decision), queue.stamped (index (decision)));
  return decision;
}

#endif

unsigned Solver::decide_variable () {
#ifndef QUIET
  assert (level || statistics.active_variables == unassigned);
#endif
#ifndef NSWITCH
  if (!stable)
    return decide_variable_in_focused_mode ();
  else
#endif
    return decide_variable_in_stable_mode ();
}

signed char Solver::decide_phase (unsigned decision) {
  signed char value = 0;
  Phase &phase = phases[decision];
#ifndef NTARGET
  if (stable && OPTION (target_phases))
    value = phase.target;
#endif
  if (!value)
    value = phase.saved;
  if (!value)
    value = OPTION (initial_phase) ? 1 : -1;
  return value;
}

void Solver::decide_literal (unsigned decision) {
  level++;
#ifndef NWALK
  if (!warming)
#endif
#ifndef NVIVIFY
    if (!vivifying)
#endif
#ifndef NPROBE
      if (!probing)
#endif
        INC_COUNT (decisions);
  if (decision == INVALID_LITERAL) {
    unsigned idx = decide_variable ();
    signed char value = decide_phase (idx);
    decision = literal (idx);
    if (value < 0)
      decision = negate (decision);
  }
  debug ("decision %s", debug_literal (decision));

  assert (level == control.size ());
  assert (trail.size () <= UINT_MAX);
  control.push_back (Frame (decision, trail.size ()));

  assign (decision, DECISION_REASON);
}

}; // namespace TabularaSAT
