#!/bin/sh

dir=`pwd|xargs basename`
prefix="test/test.sh[$dir]"

msg () {
  echo "$prefix: $*" 
}

die () {
  echo "$prefix: error: $*"
  exit 1
}

binary=../../build/$dir/binary
test -x $binary || die "could not find '$binary'"

msg "using '$binary'"

if [ ! -d log ]
then
  mkdir log || exit 1
fi

drattrim=./drat-trim
if [ ! -f $drattrim ]
then
  msg "trying to compile local copy of 'drat-trim'"
  if make drat-trim
  then
    msg "using '$drattrim'"
  else
    msg "compilation of 'drat-trim' failed"
    drattrim=""
  fi
fi

run () {
  expected=$1
  name=$2
  log=log/$name.log
  err=log/$name.err
  cnf=../../test/cnfs/$name.cnf
  test -f $cnf || die "could not find '$cnf'"
  rm -f $log $err
  cmd="$binary $cnf"
  echo $cmd
  $cmd 1>$log 2>$err
  status=$?
  test "$status" = "$expected" || \
    die "'$cmd' exit status '$status' (expected '$expected')"
  test "$expected" = 20 || return
  test "$drattrim" = "" && return
  prf=log/$name.proof
  rm -f $prf
  cmd="$binary $cnf $prf"
  echo $cmd
  $cmd 1>>$log 2>>$err
  status=$?
  test "$status" = "$expected" || \
    die "'$cmd' exit status '$status' (expected '$expected')"
  cmd="$drattrim $cnf $prf"
  echo "$cmd"
  $cmd 1>>$log 2>>$err
  test "`grep 's VERIFIED' $log`" = "" && \
    die "'$cmd' does not produce 's VERIFIED' in '$log'"
}

run 10 true
run 20 false

run 10 unit1
run 10 unit2
run 10 unit3
run 10 unit4
run 20 unit5
run 20 unit6
run 10 unit7
run 20 unit8
run 20 unit9

run 20 full1
run 20 full2
run 20 full3
run 20 full4

run 20 add4
run 20 add8

run 10 prime4
run 10 prime9
run 10 prime25
run 10 prime49
run 10 prime121
run 10 prime169
run 10 prime289
run 10 prime361
run 10 prime529
run 10 prime841
run 10 prime961
run 10 prime1849
run 10 prime1369
run 10 prime1681
run 10 prime2209

run 20 add16
run 20 add32
run 20 add64
run 20 add128

run 20 prime65537
#run 20 prime4294967297

exit 0
